/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

 /*!
   \file terralib/qt/widgets/dataset/utils/DataSetTypeFilter.h

   \brief Custom widget used filter a dataset type, creating a DataSetTypeConverter.
 */

#ifndef __TERRALIB_QT_WIDGETS_DATASET_UTILS_INTERNAL_DATASETTYPEFILTER_H
#define __TERRALIB_QT_WIDGETS_DATASET_UTILS_INTERNAL_DATASETTYPEFILTER_H

 // TerrLib
#include "../../Config.h"

#ifndef Q_MOC_RUN
#include "../../../../dataaccess/dataset/DataSetType.h"
#include "../../../../dataaccess/dataset/DataSetTypeConverter.h"
#endif

// Qt
#include <QGridLayout>
#include <QWidget>

// STL
#include <memory>

namespace te
{
  namespace qt
  {
    namespace widgets
    {
      class DoubleListWidget;

      /*!
      \class DataSetTypeFilter

      \brief Custom widget used filter a dataset type, creating a DataSetTypeConverter.

      \note If input DataSetType has primary key, the output will keep the same pk.

      \note If input DataSetType has geom prop, the output will keep the geom prop.


    */
      class TEQTWIDGETSEXPORT DataSetTypeFilter : public QWidget
      {
        Q_OBJECT

        public:

          /*! \brief Default constructor. */
          DataSetTypeFilter(QWidget* parent = 0, Qt::WindowFlags f = 0);

          /*! \brief Virtual destructor. */
          ~DataSetTypeFilter();

        public:

          /*!
            \brief Function used to set the input data set type

            \param dsType Pointer to a valid dataset type
          */
          void setDataSetType(const te::da::DataSetType* dsType);

          /*!
            \brief It gets the DataSetType converter based on attributes selected on inteface

            \param name New dataset name to use in a adapted dataset type

            \return Converter with filtered attributes
          */
          std::unique_ptr<te::da::DataSetTypeConverter> getConverter(std::string name);

        protected:

          std::unique_ptr<te::da::DataSetType> m_dsType;   //!< Input dataset type

          std::unique_ptr<te::qt::widgets::DoubleListWidget> m_doubleListWidget;
      };
    }
  }
}

#endif //__TERRALIB_QT_WIDGETS_DATASET_UTILS_INTERNAL_DATASETTYPEFILTER_H
